﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

namespace MyTWeenLib
{
    /// <summary>
    /// Tweenアニメーションを実行するクラス
    /// </summary>
    public class MoveTWeen : MonoBehaviour
    {
        readonly Queue<IEnumerator> moveTWeenQueue = new Queue<IEnumerator>();

        public UnityEvent CompleteAction = new UnityEvent();

        /// <summary>
        /// スタート時点で実行されるコルーチン
        /// キューにデータがある場合、順番に実行されるようにする
        /// キューのメソッドが完了するのを待って、次のキューのメソッドを実行
        /// コルーチンで、メソッド完了待ち
        /// </summary>
        /// <returns>イテレータ（コルーチンで必要）</returns>
        IEnumerator Start()
        {
            Debug.Log($"{gameObject.name}： MoveLinearメソッド開始");

            while (true)
            {
                switch (moveTWeenQueue.Count)
                {
                    case 0:
                        if (CompleteAction != null)
                        {
                            CompleteAction.Invoke();
                            CompleteAction = null;
                        }
                        yield return null;
                        break;

                    default:
                        Debug.Log($"{gameObject.name}： TWeen 開始");

                        yield return StartCoroutine(moveTWeenQueue.Dequeue());

                        Debug.Log($"{gameObject.name}： Tween 終了");
                        break;
                }
            }
        }

        /// <summary>
        /// 実行待ちキューにコルーチンをセット
        /// </summary>
        /// <param name="obj">移動するゲームオブジェクト</param>
        /// <param name="endPos">到達ポシション</param>
        /// <param name="moveTime">到達するまでの時間</param>
        public void MoveObj(GameObject obj, Vector3 endPos, float moveTime)
        {
            moveTWeenQueue.Enqueue(MoveLinear(obj, endPos, moveTime));
        }

        /// <summary>
        /// 移動を実施する
        /// コルーチンで、移動時間から１フレームで移動する教理を計算して、少しずつ移動するようにしている
        /// </summary>
        /// <param name="obj">移動するゲームオブジェクト</param>
        /// <param name="endPos">到達ポシション</param>
        /// <param name="moveTime">到達するまでの時間</param>
        /// <returns>イテレーター（コルーチンで必要）</returns>
        IEnumerator MoveLinear(GameObject obj, Vector3 endPos, float movetTime)
        {
            var deltaMovePos = (endPos - obj.transform.position) * Time.deltaTime / movetTime;

            float nextDiffPosSize;
            float diffPosSize;

            do
            {
                var nextPos = obj.transform.position + deltaMovePos;

                diffPosSize = (endPos - nextPos).magnitude;

                nextDiffPosSize = (endPos - (nextPos + deltaMovePos)).magnitude;

                obj.transform.position = nextPos;

                yield return null;

            } while (diffPosSize - nextDiffPosSize > 0);

            obj.transform.position = endPos;

            Debug.Log($"{gameObject.name}： TWeen終了位置：{endPos}");
        }
    }
}